import os
import glob
import unittest
import imsize


thisdir = os.path.dirname(__file__)
imagedir = os.path.join(thisdir, "images")


class ReadTest(unittest.TestCase):

    def test_png(self):
        pngs = glob.glob(os.path.join(imagedir, "*.png"))
        self.assertTrue(len(pngs) > 0)
        for i, png in enumerate(sorted(pngs)):
            info = imsize.read(png)
            self.assertEqual(info.filetype, "png")
            self.assertEqual(info.nchan, 3)
            self.assertTrue(info.width in [600, 450])
            self.assertTrue(info.height in [600, 450])
            self.assertEqual(info.bitdepth, 8)
            self.assertEqual(info.bytedepth, 1)
            self.assertEqual(info.maxval, 255)
            self.assertEqual(info.isfloat, False)
            self.assertEqual(info.uncertain, False)
            self.assertEqual(info.cfa_raw, False)
            self.assertEqual(info.nbytes, 600 * 450 * 3)
            self.assertEqual(info.orientation, 0)

    def test_bmp(self):
        bmps = glob.glob(os.path.join(imagedir, "*.bmp"))
        self.assertTrue(len(bmps) > 0)
        for i, bmp in enumerate(sorted(bmps)):
            info = imsize.read(bmp)
            self.assertEqual(info.filetype, "bmp")
            self.assertEqual(info.nchan, 3)
            self.assertTrue(info.width, 640)
            self.assertTrue(info.height, 426)
            self.assertEqual(info.bitdepth, 8)
            self.assertEqual(info.bytedepth, 1)
            self.assertEqual(info.maxval, 255)
            self.assertEqual(info.isfloat, False)
            self.assertEqual(info.uncertain, False)
            self.assertEqual(info.cfa_raw, False)
            self.assertEqual(info.nbytes, 640 * 426 * 3)
            self.assertEqual(info.orientation, 0)

    def test_dng(self):
        dngs = glob.glob(os.path.join(imagedir, "*.DNG"))
        self.assertTrue(len(dngs) > 0)
        for i, dng in enumerate(sorted(dngs)):
            info = imsize.read(dng)
            self.assertEqual(info.filetype, "dng")
            self.assertEqual(info.nchan, 1)
            self.assertTrue(info.width, 7296)
            self.assertTrue(info.height, 3648)
            self.assertEqual(info.bitdepth, 12)
            self.assertEqual(info.bytedepth, 2)
            self.assertEqual(info.maxval, 4095)
            self.assertEqual(info.isfloat, False)
            self.assertEqual(info.uncertain, False)
            self.assertEqual(info.cfa_raw, True)
            self.assertEqual(info.nbytes, 7296 * 3648 * 2)
            self.assertEqual(info.orientation, 0)

    def test_exr(self):
        exrs = glob.glob(os.path.join(imagedir, "*.exr"))
        self.assertTrue(len(exrs) > 0)
        for i, exr in enumerate(sorted(exrs)):
            info = imsize.read(exr)
            self.assertEqual(info.filetype, "exr")
            self.assertEqual(info.isfloat, True)
            self.assertEqual(info.cfa_raw, False)
            self.assertEqual(info.width, 800)
            self.assertEqual(info.height, 800)
            self.assertEqual(info.nchan, 1)
            self.assertEqual(info.bitdepth, 16)
            self.assertEqual(info.bytedepth, 2)

    def test_npy(self):
        npys = glob.glob(os.path.join(imagedir, "*.npy"))
        self.assertTrue(len(npys) > 0)
        for i, npy in enumerate(sorted(npys)):
            info = imsize.read(npy)
            self.assertEqual(info.filetype, "npy")
            self.assertEqual(info.isfloat, "float" in npy)
            self.assertEqual(info.width, 20)
            self.assertEqual(info.height, 10)
            self.assertEqual(info.nchan, 5)
            self.assertEqual(info.bitdepth, 16)
            self.assertEqual(info.bytedepth, 2)
            self.assertEqual(info.nbytes, 20 * 10 * 5 * 2)

    def test_orientations(self):
        jpegs = sorted(glob.glob(os.path.join(imagedir, "orientations", "*.jpg")))
        tiffs = sorted(glob.glob(os.path.join(imagedir, "orientations", "*.tif")))
        self.assertTrue(len(jpegs) == 16)
        self.assertTrue(len(tiffs) == 16)
        for fmt, fileset in zip(["jpeg", "tiff"], [jpegs, tiffs]):
            for i, filespec in enumerate(fileset):
                info = imsize.read(filespec)
                self.assertEqual(info.filetype, fmt)
                self.assertEqual(info.nchan, 3)
                self.assertTrue(info.width in [600, 450])
                self.assertTrue(info.height in [600, 450])
                self.assertEqual(info.bitdepth, 8)
                self.assertEqual(info.bytedepth, 1)
                self.assertEqual(info.maxval, 255)
                self.assertEqual(info.isfloat, False)
                self.assertEqual(info.uncertain, False)
                self.assertEqual(info.cfa_raw, False)
                self.assertEqual(info.nbytes, 600 * 450 * 3)
                self.assertEqual(info.orientation, (i % 8) + 1)


if __name__ == "__main__":
    unittest.main()



"""
A minimal header parser for PFM images, meant solely for querying the
dimensions and bit depth.
"""

import re  # built-in library


######################################################################################
#
#  P U B L I C   A P I
#
######################################################################################


def dims(filespec, verbose=False):
    """
    Returns the dimensions (width, height, and number of channels) of the given
    PFM file. Also returns the nominal scale of the pixel values (typically 1.0).
    """
    with open(filespec, "rb") as f:
        header = f.read(64)  # should be enough for any valid header
        shape, scale = __parse_header(header, filespec, verbose)
        return (shape, scale)


######################################################################################
#
#  I N T E R N A L   F U N C T I O N S
#
######################################################################################


def __parse_header(header, filespec, verbose=False):
    regex_pfm_header = b"(^(P[Ff])\\s+(\\d+)\\s+(\\d+)\\s+([+-]?\\d+(?:\\.\\d+)?)\\s)"
    match = re.search(regex_pfm_header, header)
    if match is not None:
        header, typestr, width, height, scale = match.groups()
        width, height, scale = int(width), int(height), float(scale)
        numch = 3 if typestr == b"PF" else 1
        shape = (height, width, numch) if numch == 3 else (height, width)
        dtype = "<f" if scale < 0.0 else ">f"
        scale = abs(scale)
        if verbose:
            print(f"Reading file {filespec} ", end='')
            print(f"(w={width}, h={height}, c={numch}, scale={scale:.3f}, byteorder='{dtype[0]}')")
        return (shape, scale)
    raise RuntimeError(f"File {filespec} is not a valid PFM file.")



#!/usr/bin/env -S python3 -B

"""
A collection of simple command-line parsing functions.

Copyright (c) 2018 Tomi Aarnio. MIT License.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import os                         # standard library
import sys                        # standard library
import re                         # standard library # pylint: disable=unused-import # noqa
import glob                       # standard library
import unittest                   # standard library

# pylint: disable=invalid-name


######################################################################################
#
#  P U B L I C   A P I
#
######################################################################################


def filenames(patterns, extensions=None, sort=False, allowAllCaps=False, numRequired=0):
    """
    Examples:
      filenames, basenames = argv.filenames(sys.argv[1:], numRequired=2)
      filenames, basenames = argv.filenames(sys.argv[1:], [".ppm", ".png"], sort=True)
      filenames, basenames = argv.filenames(sys.argv[1:], [".jpg"], allowAllCaps=True)
    """
    fullnames = [glob.glob(filepattern, recursive=True) for filepattern in patterns]  # expand wildcards
    fullnames = [item for sublist in fullnames for item in sublist]                 # flatten nested lists
    fullnames = [f for f in set(fullnames) if os.path.isfile(f)]                    # check file existence
    if extensions is not None:
        extensions += [e.upper() for e in extensions] if allowAllCaps else []       # jpg => [jpg, JPG]
        fullnames = [f for f in fullnames if os.path.splitext(f)[1] in extensions]  # filter by extension
    fullnames = sorted(fullnames) if sort else fullnames                            # sort if requested
    basenames = [os.path.splitext(f)[0] for f in fullnames]                         # strip extensions
    errmsg = f"Found {len(fullnames)} matching files, {numRequired} required."      # check file count
    _enforce(len(fullnames) >= numRequired, errmsg)
    return fullnames, basenames


def exists(argname):
    """
    Example:
      showHelp = argv.exists("--help")
    """
    if argname in sys.argv:
        argidx = sys.argv.index(argname)
        del sys.argv[argidx]
        return True
    return False


def intval(argname, default=None, accepted=None, condition=None, repeats=False):
    """
    Example:
      numtiles = argv.intval("--split", 1, [1, 2, 3, 4])
    """
    if not repeats:  # pylint: disable=no-else-return
        argstr = _string(argname)
        useDefault = argstr is None
        if not useDefault:
            errmsg = f"Invalid value for '{argname}': '{argstr}' does not represent an integer."
            _enforce(_isInt(argstr), errmsg)
            argval = int(argstr, 0)  # hex values must have the "0x" prefix for this to work
            if not _isValid(argname, argval, accepted, condition):
                sys.exit(-1)
        return default if useDefault else argval
    else:
        argSet = set()
        argstr = _string(argname)
        if argstr is None:
            argSet = argSet.union([default])
        while argstr is not None:
            errmsg = f"Invalid value for '{argname}': '{argstr}' does not represent an integer."
            _enforce(_isInt(argstr), errmsg)
            argval = int(argstr, 0)  # hex values must have the "0x" prefix for this to work
            if not _isValid(argname, argval, accepted, condition):
                sys.exit(-1)
            argSet = argSet.union([argval])
            argstr = _string(argname)
        return argSet


def floatval(argname, default=None, accepted=None, condition=None):
    """
    Example:
      factor = argv.floatval("--factor", 1.0, condition='1.0 <= v <= 3.0')
    """
    argstr = _string(argname)
    useDefault = argstr is None
    if not useDefault:
        errmsg = f"Invalid value for '{argname}': '{argstr}' does not represent a number."
        _enforce(_isFloat(argstr), errmsg)
        if not _isValid(argname, float(argstr), accepted, condition):
            sys.exit(-1)
    return default if useDefault else float(argstr)


def stringval(argname, default=None, accepted=None, condition=None, repeats=False):
    """
    Examples:
      bayer = argv.stringval("--bayer", default="AUTO", accepted=["AUTO", "GBRG", "RGGB"])
      sources = argv.stringval("--source", default="all", accepted=["foo", "bar", "baz", "all"], repeats=True)
    """
    if not repeats:  # pylint: disable=no-else-return
        argstr = _string(argname)
        useDefault = argstr is None
        if not useDefault:
            if not _isValid(argname, argstr, accepted, condition):
                sys.exit(-1)
        return default if useDefault else argstr
    else:
        argSet = set()
        argstr = _string(argname)
        if argstr is None:
            argSet |= set([default])
        while argstr is not None:
            if not _isValid(argname, argstr, accepted, condition):
                sys.exit(-1)
            argSet = argSet.union([argstr])
            argstr = _string(argname)
        return argSet


def intpair(argname, default=None, repeats=False):
    """
    Examples:
      width, height = argv.intpair("--size", default=(1920, 1080))
      limits = argv.intpair("--limits", default=(0, 1), repeats=True)
    """
    if not repeats:
        pair = _intpair(argname)
        pair = default if pair is None else pair
        return pair
    else:
        pairSet = set()
        pair = _intpair(argname)
        if pair is None:
            pairSet |= set([default])
        while pair is not None:
            pairSet |= set([pair])
            pair = _intpair(argname)
        return pairSet


def floatpair(argname, default=None):
    """
    Example:
      factor1, factor2 = argv.floatpair("--factors", default=(1.0, 1.0))
    """
    if argname in sys.argv:
        argidx = sys.argv.index(argname)
        if len(sys.argv) >= argidx + 3:
            val1 = sys.argv[argidx + 1]
            val2 = sys.argv[argidx + 2]
            del sys.argv[argidx:argidx + 3]
            _enforce(_isFloat(val1), f"Invalid value for '{argname}': '{val1}' does not represent a decimal number")
            _enforce(_isFloat(val2), f"Invalid value for '{argname}': '{val2}' does not represent a decimal number.")
            val1 = float(val1)
            val2 = float(val2)
            return [val1, val2]
        print(f"Missing value(s) for '{argname}': Expected a pair of decimal numbers.")
        sys.exit(-1)
    return default


def floatstring(argname, default=None, accepted=None):
    """
    Examples:
      blacklevel = argv.floatstring("--whitelevel", "AUTO", accepted=["AUTO"])
      whitelevel = argv.floatstring("--blacklevel", 1023.0, accepted=["AUTO"])
    """
    argstr = _string(argname)
    if argstr is not None:
        try:
            result = float(argstr)
            return result
        except ValueError:
            if _isValid(argname, argstr, accepted):
                return argstr
            sys.exit(-1)
    else:
        return default


def exitIfAnyUnparsedOptions():
    """
    Example:
      factor = argv.floatval("--factor", default=1.0)
      showHelp = argv.exists("--help")
      argv.exitIfAnyUnparsedOptions()
    """
    isOptionArg = [arg.startswith("-") for arg in sys.argv]
    if any(isOptionArg):
        argname = sys.argv[isOptionArg.index(True)]
        print(f"Unrecognized command-line option: {argname}")
        sys.exit(-1)


######################################################################################
#
#  I N T E R N A L   F U N C T I O N S
#
######################################################################################


def _enforce(expression, errorMessageIfFalse):
    """ If 'expression' is False, prints out the given error message and exits. """
    if not expression:
        print(errorMessageIfFalse)
        sys.exit(-1)


def _string(argname, default=None):
    """ Parses arguments of the form '--argname string', returns the string. """
    if argname in sys.argv:
        argidx = sys.argv.index(argname)
        if len(sys.argv) >= argidx + 2:
            argstr = sys.argv[argidx + 1]
            del sys.argv[argidx:argidx + 2]
            return argstr
        else:
            return ""
    return default


def _isInt(argstr):
    """ Returns True if and only if the given string represents an integer. """
    try:
        int(argstr, 0)  # hex values must have the "0x" prefix for this to work
        return True
    except (ValueError, TypeError):
        return False


def _isFloat(argstr):
    """ Returns True if and only if the given string represents a float. """
    try:
        float(argstr)
        return True
    except ValueError:
        return False


def _isValid(argname, arg, validArgs=None, condition=None):
    """ Checks that 'arg' is in validArgs and satisfies the given condition. """
    if validArgs is not None:
        if arg not in validArgs:
            print(f"Invalid value for '{argname}': '{arg}' is not in the set {validArgs}.")
            return False
    if condition is not None:
        validator = eval(f"lambda v: {condition}")  # pylint: disable=eval-used
        if validator(arg) is not True:
            print(f"Invalid value for '{argname}': '{arg}' does not satisfy '{condition}'.")
            return False
    return True


def _intpair(argname):
    if argname in sys.argv:
        argidx = sys.argv.index(argname)
        if len(sys.argv) >= argidx + 3:
            val1 = sys.argv[argidx + 1]
            val2 = sys.argv[argidx + 2]
            del sys.argv[argidx:argidx + 3]
            _enforce(_isInt(val1), f"Invalid value for '{argname}': '{val1}' does not represent an integer.")
            _enforce(_isInt(val2), f"Invalid value for '{argname}': '{val2}' does not represent an integer.")
            val1 = int(val1)
            val2 = int(val2)
            return (val1, val2)
        print(f"Missing value(s) for '{argname}': Expected a pair of integers.")
        sys.exit(-1)
    return None


######################################################################################
#
#  U N I T   T E S T S
#
######################################################################################


class _Tests(unittest.TestCase):

    # pylint: disable=missing-docstring

    def test_exists(self):
        print("Testing argv.exists()...")
        sys.argv = ["argv.py", "--foo"]
        self.assertEqual(exists("--foo"), True)
        self.assertEqual(exists("--foo"), False)
        exitIfAnyUnparsedOptions()

    def test_intval(self):
        print("Testing argv.intval()...")
        sys.argv = ["--foo", "2", "--bar", "4", "--baz", "0xfe"]
        self.assertEqual(intval("--foo"), 2)
        self.assertEqual(intval("--baz"), 254)
        self.assertEqual(intval("--bar", accepted=[3, 4, 5]), 4)
        self.assertEqual(exists("--foo"), False)
        exitIfAnyUnparsedOptions()

    def test_floatval(self):
        print("Testing argv.floatval()...")
        sys.argv = ["--foo", "2", "--bar", "0.3"]
        self.assertEqual(floatval("--foo"), 2.0)
        self.assertEqual(floatval("--bar", condition='v >= 0.2'), 0.3)
        self.assertEqual(exists("--foo"), False)
        exitIfAnyUnparsedOptions()

    def test_stringval(self):
        print("Testing argv.stringval()...")
        sys.argv = ["--foo", "2", "--bar", "baz", "--baz", "foo"]
        self.assertEqual(stringval("--foo"), "2")
        self.assertEqual(stringval("--bar", condition="len(v) > 2"), "baz")
        self.assertEqual(stringval("--baz", accepted=["foo"]), "foo")
        exitIfAnyUnparsedOptions()

    def test_stringval_repeat(self):
        print("Testing argv.stringval(repeats=True)...")
        sys.argv = ["--repeated", "foo", "--repeated", "bar", "--repeated", "baz"]
        self.assertEqual(stringval("--repeated", repeats=True), set(("foo", "bar", "baz")))
        exitIfAnyUnparsedOptions()

    def test_intpair(self):
        print("Testing argv.intpair()...")
        sys.argv = ["--foo", "-1", "1"]
        self.assertEqual(intpair("--foo"), (-1, 1))
        self.assertEqual(intpair("--foo", default=(0, 2)), (0, 2))
        sys.argv = ["--repeated", "-1", "1", "--repeated", "10", "20"]
        self.assertEqual(intpair("--repeated", repeats=True), set([(-1, 1), (10, 20)]))
        self.assertEqual(intpair("--repeated", default=(2, 1), repeats=True), set([(2, 1)]))

    def test_floatpair(self):
        print("Testing argv.floatpair()...")
        sys.argv = ["--foo", "-1.2", "2.1"]
        self.assertEqual(floatpair("--foo"), [-1.2, 2.1])
        self.assertEqual(intpair("--foo", default=[0.1, -1.5]), [0.1, -1.5])

    def test_missing_values(self):
        print("Testing missing argument values...")
        sys.argv = ["--foo", "3"]
        self.assertRaises(SystemExit, lambda: intpair("--foo"))
        sys.argv = ["--foo", "3.0"]
        self.assertRaises(SystemExit, lambda: floatpair("--foo"))
        sys.argv = ["--foo", "3.1", "3.2"]
        self.assertRaises(SystemExit, lambda: intpair("--foo"))
        sys.argv = ["--foo", "2", "2.1"]
        self.assertRaises(SystemExit, lambda: intpair("--foo"))
        sys.argv = ["--foo"]
        self.assertRaises(SystemExit, lambda: stringval("--foo", accepted=["bar"]))

    def test_invalid_types(self):
        print("Testing invalid numeric types...")
        sys.argv = ["--foo", "2.0", "--bar", "-5.0", "--foo", "0xfg", "--baz", "-.1e", "--bae"]
        self.assertRaises(SystemExit, lambda: intval("--foo"))
        self.assertRaises(SystemExit, lambda: intval("--bar"))
        self.assertRaises(SystemExit, lambda: intval("--foo"))
        self.assertRaises(SystemExit, lambda: floatval("--baz"))
        self.assertRaises(SystemExit, exitIfAnyUnparsedOptions)

    def test_conditions(self):
        print("Testing invalid numeric values...")
        sys.argv = ["--foo", "2.0", "--bar", "-5.0", "--str1", "baz", "--str2", "foo"]
        self.assertRaises(SystemExit, lambda: floatval("--foo", condition="v > 2.0"))
        self.assertRaises(SystemExit, lambda: floatval("--bar", accepted=[-4.99, -5.01, -5.02]))
        self.assertRaises(SystemExit, lambda: stringval("--str1", accepted=["foo", "bar"]))
        self.assertRaises(SystemExit, lambda: stringval("--str2", condition="len(v) > 4"))
        exitIfAnyUnparsedOptions()


def __main():
    print("--" * 35)
    suite = unittest.TestLoader().loadTestsFromTestCase(_Tests)
    unittest.TextTestRunner(verbosity=0).run(suite)


if __name__ == "__main__":
    __main()



#!/usr/bin/python3 -B

"""
Extracts image dimensions, bit depth, and other basic metadata.
"""

import os              # built-in library
import math            # built-in library
import struct          # built-in library
import ast             # built-in library
import pprint          # built-in library
import piexif          # pip install piexif
import pyexiv2         # pip install pyexiv2
import rawpy           # pip install rawpy
import numpy as np     # pip install numpy

try:
    # package mode
    from imsize import pnmhdr   # local import: pnmhdr.py
    from imsize import pfmhdr   # local import: pfmhdr.py
    from imsize import exrhdr   # local import: exrhdr.py
except ImportError:
    # stand-alone mode
    import pnmhdr
    import pfmhdr
    import exrhdr


######################################################################################
#
#  P U B L I C   A P I
#
######################################################################################


class ImageInfo:
    """
    A container for image metadata, filled in and returned by read().

    Attributes:
      filespec (str): The filespec given to read(), copied verbatim
      filetype (str): File type: png|pnm|pfm|bmp|jpeg|insp|tiff|exr|dng|cr2|nef|raw|npy
      filesize (int): Size of the file on disk in bytes
      header_size (int): Size of .raw file header in bytes
      isfloat (bool): True if the image is in floating-point format
      cfa_raw (bool): True if the image is in CFA (Bayer) raw format
      width (int): Width of the image in pixels (orientation ignored)
      height (int): Height of the image in pixels (orientation ignored)
      nchan (int): Number of color channels: 1, 2, 3 or 4
      bitdepth (int): Bits per sample: 1 to 32
      bytedepth (int): Bytes per sample: 1, 2 or 4
      maxval (int): Maximum representable sample value, e.g., 255
      nbytes (int): Size of the image in bytes, uncompressed
      orientation (int): Image orientation in EXIF format: 1 to 8, or 0 (unknown)
      rot90_ccw_steps (int): Number of rotations to bring the image upright: 0 to 3
      uncertain (bool): True if width/height/bitdepth are uncertain
    """
    def __init__(self):
        self.filespec = None
        self.filetype = None
        self.filesize = None
        self.header_size = None
        self.isfloat = None
        self.cfa_raw = None
        self.width = None
        self.height = None
        self.nchan = None
        self.bitdepth = None
        self.bytedepth = None
        self.maxval = None
        self.nbytes = None
        self.orientation = None
        self.rot90_ccw_steps = None
        self.uncertain = None

    def __repr__(self):
        reprstr = f"<ImageInfo {self.__dict__}>"
        return reprstr

    def __str__(self):
        infostr = pprint.pformat(self.__dict__, sort_dicts=False)
        return infostr


def read(filespec):
    """
    Parses a lowest common denominator set of metadata from the given
    image, i.e., the dimensions and bit depth. Does not read the entire
    file but only what's necessary. Returns an ImageInfo with all fields
    filled in, or None in case of failure.

    Example:
      info = imsize.read("myfile.jpg")
      factor = info.nbytes / info.filesize
      print(f"{info.filespec}: compression factor = {factor.1f}")
    """
    filename = os.path.basename(filespec)             # "path/image.ext" => "image.ext"
    extension = os.path.splitext(filename)[-1]        # "image.ext" => ("image", ".ext")
    filetype = extension.lower()[1:]                  # ".EXT" => "ext"
    handlers = {"png": _read_png,
                "pnm": _read_pnm,
                "pgm": _read_pnm,
                "ppm": _read_pnm,
                "pfm": _read_pfm,
                "bmp": _read_bmp,
                "jpeg": _read_jpeg,
                "jpg": _read_jpeg,
                "insp": _read_insp,
                "tiff": _read_tiff,
                "tif": _read_tiff,
                "exr": _read_exr,
                "dng": _read_dng,
                "cr2": _read_cr2,
                "nef": _read_nef,
                "raw": _read_raw,
                "npy": _read_npy}
    if filetype in handlers:
        handler = handlers[filetype]
        info = handler(filespec)
        return info
    return None


######################################################################################
#
#  I N T E R N A L   F U N C T I O N S
#
######################################################################################


def _read_png(filespec):
    info = ImageInfo()
    info.filespec = filespec
    info.filetype = "png"
    info.isfloat = False
    info.cfa_raw = False
    with open(filespec, "rb") as f:
        header = f.read(26)
        signature = bytes([0x89, 0x50, 0x4e, 0x47, 0x0d, 0x0a, 0x1a, 0x0a])
        if header.startswith(signature) and header[12:16] == b"IHDR":
            ihdr = struct.unpack(">LLBB", header[16:26])
        info.width = ihdr[0]
        info.height = ihdr[1]
        info.bitdepth = ihdr[2]
        info.nchan = {0: 1,           # greyscale => 1 channel
                      2: 3,           # truecolor => 3 channels
                      3: 3,           # indexed => 3 channels
                      4: 2,           # greyscale_alpha => 2 channels
                      6: 4}[ihdr[3]]  # truecolor_alpha => 4 channels
        info = _complete(info)
        return info


def _read_pnm(filespec):
    info = ImageInfo()
    shape, maxval = pnmhdr.dims(filespec)
    info.filespec = filespec
    info.filetype = "pnm"
    info.isfloat = False
    info.cfa_raw = False
    info.width = shape[1]
    info.height = shape[0]
    info.nchan = 1 if len(shape) < 3 else shape[2]
    info.maxval = maxval
    info = _complete(info)
    return info


def _read_pfm(filespec):
    info = ImageInfo()
    shape, maxval = pfmhdr.dims(filespec)
    info.filespec = filespec
    info.filetype = "pfm"
    info.isfloat = True
    info.cfa_raw = False
    info.width = shape[1]
    info.height = shape[0]
    info.nchan = 1 if len(shape) < 3 else shape[2]
    info.maxval = maxval
    info.bitdepth = 32
    info.bytedepth = 4
    info = _complete(info)
    return info


def _read_bmp(filespec):
    info = ImageInfo()
    info.filespec = filespec
    info.filetype = "bmp"
    info.isfloat = False
    info.cfa_raw = False
    with open(filespec, "rb") as f:
        bmp_header = f.read(14)
        if bmp_header[:2] == b"BM":
            dib_header = struct.unpack("<LLLHH", f.read(4 + 4 + 4 + 2 + 2))
            info.width = dib_header[1]
            info.height = dib_header[2]
            bpp = dib_header[4]
            info.nchan = {1: 1,        # 1 bpp => 1 channel
                          2: 3,        # 2 bpp palettized => 3 channels
                          4: 3,        # 4 bpp palettized => 3 channels
                          8: 3,        # 8 bpp palettized => 3 channels
                          16: 4,       # 16 bpp RGBA => 4 channels
                          24: 3,       # 24 bpp RGB => 3 channels
                          32: 4}[bpp]  # 32 bpp RGBA => 4 channels
            info.maxval = 255
        info = _complete(info)
        return info


def _read_exr(filespec):
    info = ImageInfo()
    info.filespec = filespec
    info.filetype = "exr"
    info.cfa_raw = False
    info.maxval = 1.0
    w, h, nchan, isfloat, bitdepth = exrhdr.dims(filespec)
    info.width = w
    info.height = h
    info.nchan = nchan
    info.isfloat = isfloat
    info.bitdepth = bitdepth
    info.bytedepth = bitdepth // 8
    info = _complete(info)
    return info


def _read_jpeg(filespec):
    info = _read_exif_orientation(filespec)
    info.filespec = filespec
    info.filetype = "jpeg"
    info.isfloat = False
    info.cfa_raw = False
    with open(filespec, "rb") as f:
        if f.read(2) == b"\xff\xd8":
            size = 2
            segtype = 0
            while not 0xc0 <= segtype <= 0xcf or segtype in [0xc4, 0xc8, 0xcc]:
                f.seek(size - 2, 1)  # skip to next segment
                _0xff, segtype, size = struct.unpack(">BBH", f.read(4))
            sof = struct.unpack(">BHHB", f.read(6))
            info.bitdepth = sof[0]
            info.height = sof[1]
            info.width = sof[2]
            info.nchan = sof[3]
            info = _complete(info)
    return info


def _read_insp(filespec):
    info = _read_jpeg(filespec)
    info.filetype = "insp"
    return info


def _read_exif_orientation(filespec):
    info = ImageInfo()
    exif = piexif.load(filespec)
    exif = exif.pop("0th")
    info.orientation = exif.get(piexif.ImageIFD.Orientation)
    exif_to_rot90 = {1: 0, 2: 0, 3: 2, 4: 0, 5: 1, 6: 3, 7: 3, 8: 1}
    if info.orientation in exif_to_rot90:
        info.rot90_ccw_steps = exif_to_rot90[info.orientation]
    return info


def _read_exif(filespec):
    info = ImageInfo()
    info.filespec = filespec
    info.filetype = "exif"
    info.isfloat = False
    img = pyexiv2.Image(filespec)
    exif = img.read_exif()
    subimages = ["Image", "SubImage1", "SubImage2", "SubImage3"]
    widths = [exif.get(f"Exif.{sub}.ImageWidth") for sub in subimages]
    widths = [int(w or 0) for w in widths]  # None => 0
    maximg = subimages[np.argmax(widths)]  # use the largest sub-image
    info.cfa_raw = exif.get(f"Exif.{maximg}.PhotometricInterpretation") in ['32803', '34892']
    info.width = int(exif.get(f"Exif.{maximg}.ImageWidth"))
    info.height = int(exif.get(f"Exif.{maximg}.ImageLength"))
    info.nchan = int(exif.get(f"Exif.{maximg}.SamplesPerPixel"))
    info.bitdepth = exif.get(f"Exif.{maximg}.BitsPerSample")
    info.orientation = int(exif.get(f"Exif.{maximg}.Orientation") or 0)
    exif_to_rot90 = {1: 0, 2: 0, 3: 2, 4: 0, 5: 1, 6: 3, 7: 3, 8: 1}
    if info.orientation in exif_to_rot90:
        info.rot90_ccw_steps = exif_to_rot90[info.orientation]
    bitdepths = tuple(int(el) for el in info.bitdepth.split(" "))  # string => tuple of ints
    info.bitdepth = bitdepths[0]
    info = _complete(info)
    return info


def _read_tiff(filespec):
    info = _read_exif(filespec)
    info.filetype = "tiff"
    return info


def _read_dng(filespec):
    info = _read_exif(filespec)
    info.filetype = "dng"
    return info


def _read_cr2(filespec):
    exif = piexif.load(filespec)
    exif = exif.pop("0th")
    info = ImageInfo()
    info.filespec = filespec
    info.filetype = "cr2"
    info.uncertain = True
    info.isfloat = False
    info.cfa_raw = True
    info.width = exif.get(piexif.ImageIFD.ImageWidth)
    info.height = exif.get(piexif.ImageIFD.ImageLength)
    info.nchan = 1
    info.bytedepth = 2
    info.bitdepth = 14  # 14-bit since Canon 450D (2008)
    info = _complete(info)
    return info


def _read_nef(filespec):  # reading the whole file ==> SLOW
    raw = rawpy.imread(filespec)
    raw = raw.raw_image_visible
    info = ImageInfo()
    info.filespec = filespec
    info.filetype = "nef"
    info.uncertain = True
    info.isfloat = False
    info.cfa_raw = True
    info.nchan = 1
    info.bytedepth = 2
    info.height, info.width = raw.shape
    minbits = np.ceil(np.log2(np.max(raw)))  # 5, 6, 7, ..., 16
    minbits = np.ceil(minbits / 2) * 2  # 6, 8, 10, 12, ..., 16
    minbits = max(minbits, 12)  # 12, 14, 16
    info.bitdepth = int(minbits)  # can underestimate bpp if image is very dark
    info = _complete(info)
    return info


def _read_raw(filespec):  # reading the whole file ==> SLOW
    info = ImageInfo()
    info.filespec = filespec
    info.filetype = "raw"
    info.uncertain = True  # width, height & bitdepth are guessed
    info.isfloat = False
    info.cfa_raw = True
    info.nchan = 1
    info.bytedepth = 2  # all sensors are at least 10-bit these days
    info.header_size = 0  # assume no header
    info.filesize = os.path.getsize(filespec)
    for aspect in [3/4, 2/3, 9/16]:  # try some typical aspect ratios
        numpixels = info.filesize / info.bytedepth
        info.height = math.sqrt(numpixels * aspect)
        info.width = numpixels / info.height
        wrem4 = info.width % 4
        hrem4 = info.height % 4
        if wrem4 == 0 and hrem4 == 0:
            info.width = int(info.width)
            info.height = int(info.height)
            break
        if wrem4 < 0.5 and hrem4 < 0.5:
            info.width = int(info.width)
            info.height = int(info.height)
            info.header_size = info.filesize - info.width * info.height * info.bytedepth
            break
        info.width = None
        info.height = None
    if info.width is not None:
        raw = np.fromfile(filespec, dtype='<u2', offset=info.header_size)  # assume x86 byte order
        minbits = np.ceil(np.log2(np.max(raw)))  # 5, 6, 7, ..., 16
        minbits = np.ceil(minbits / 2) * 2  # 6, 8, 10, 12, ..., 16
        minbits = max(minbits, 10)  # 10, 12, 14, 16
        info.bitdepth = int(minbits)  # can underestimate bpp if image is very dark
        info = _complete(info)
    else:
        info = None
    return info


def _read_npy(filespec):
    info = ImageInfo()
    info.filespec = filespec
    info.filetype = "npy"
    info.cfa_raw = False
    with open(filespec, "rb") as npyfile:
        magic = npyfile.read(6)
        assert magic == b"\x93NUMPY"
        _ = npyfile.read(2)  # version number; ignore
        header_size, = struct.unpack("<h", npyfile.read(2))
        header = npyfile.read(header_size)
        meta = ast.literal_eval(header.decode("utf-8"))
        dtype = np.dtype(meta["descr"])
        shape = meta["shape"]
        info.height, info.width = shape[:2]
        info.nchan = 1 if len(shape) < 3 else shape[2]
        info.isfloat = np.issubdtype(dtype, np.floating)
        info.bytedepth = dtype.itemsize
        info.bitdepth = info.bytedepth * 8
        info.maxval = 1.0 if info.isfloat else 2 ** info.bitdepth - 1
        info = _complete(info)
    return info


def _complete(info):
    info.filesize = info.filesize or os.path.getsize(info.filespec)
    info.maxval = info.maxval or 2 ** info.bitdepth - 1
    info.bitdepth = info.bitdepth or int(math.log2(info.maxval + 1))
    info.bytedepth = info.bytedepth or (2 if info.maxval > 255 else 1)
    info.nbytes = info.width * info.height * info.nchan * info.bytedepth
    info.uncertain = False if info.uncertain is None else info.uncertain
    info.orientation = info.orientation or 0  # None => 0
    info.rot90_ccw_steps = info.rot90_ccw_steps or 0  # None => 0
    info.header_size = info.header_size or 0  # None => 0
    return info



"""
A minimal header parser for OpenEXR images, meant solely for querying the
dimensions and bit depth.
"""

import numpy as np  # pip install numpy


######################################################################################
#
#  P U B L I C   A P I
#
######################################################################################


def dims(filespec, verbose=False):
    """
    Returns the dimensions (width, height, number of channels, datatype, and bit depth)
    of the given EXR file.
    """
    with open(filespec, "rb") as f:
        if f.read(4) == b"\x76\x2f\x31\x01":  # EXR magic number
            version = np.frombuffer(f.read(4), dtype="<u4")[0]
            max_strlen = 256 if (version & 0x400) else 32
            got_channels = False
            got_dims = False
            while not (got_channels and got_dims):
                attr_name = _read_string_nul(f, max_strlen)
                _ = _read_string_nul(f, max_strlen)  # attr_type
                attr_size = np.frombuffer(f.read(4), dtype="<u4")[0]
                if attr_name == "channels":
                    nchan = 0
                    isfloat = False
                    bitdepth = 16
                    while not got_channels:
                        name = _read_string_nul(f, max_strlen)
                        if len(name) >= 1:
                            dtype = np.frombuffer(f.read(16), dtype="<u4")[0]
                            isfloat = isfloat or (dtype > 0)
                            bitdepth = max(bitdepth, 16 if dtype == 1 else 32)
                            nchan += 1
                        else:
                            got_channels = True
                elif attr_name == "dataWindow":
                    box = np.frombuffer(f.read(16), dtype="<i4")
                    xmin, ymin, xmax, ymax = box
                    width = xmax - xmin + 1
                    height = ymax - ymin + 1
                    got_dims = True
                else:
                    _ = f.seek(attr_size, 1)
            if verbose:
                print(f"Reading file {filespec} ", end='')
                print(f"(w={width}, h={height}, c={nchan}, bitdepth={bitdepth})")
            return width, height, nchan, isfloat, bitdepth
    raise RuntimeError(f"File {filespec} is not a valid EXR file.")


######################################################################################
#
#  I N T E R N A L   F U N C T I O N S
#
######################################################################################


def _read_string_nul(f, maxlen):
    cur = f.tell()
    chunk = f.read(maxlen)
    asbytes = chunk.split(b"\x00")[0]
    string = asbytes.decode("utf-8")
    cur += len(string) + 1
    f.seek(cur)
    return string



from setuptools import setup, find_packages

import imsize


def read_deps(filename):
    with open(filename) as f:
        deps = f.read().split('\n')
        deps.remove("")
    return deps


setup(name='imsize',
      version=imsize.__version__,
      description='Lightning-fast extraction of image dimensions & bit depth.',
      url='http://github.com/toaarnio/imsize',
      author='Tomi Aarnio',
      author_email='tomi.p.aarnio@gmail.com',
      license='MIT',
      py_modules=["imsize"],
      test_suite="test",
      packages=find_packages(),
      install_requires=read_deps("requirements.txt"),
      entry_points={'console_scripts': ['imsize = imsize.consoleapp:main']},
      zip_safe=True)



#!/usr/bin/python3 -B

"""
Calculates the combined in-memory size of the given images by parsing
their header information. Runs very fast because only the header part
of a file is read from disk (except for headerless Camera RAW and Nikon
NEF).
"""

import os              # built-in library
import sys             # built-in library
import glob            # built-in library
import imsize          # pip install imsize

try:
    # package mode
    from imsize import argv
except ImportError:
    # stand-alone mode
    import argv


FILETYPES = ["*.png", "*.pnm", "*.pgm", "*.ppm", "*.pfm", "*.bmp",
             "*.jpeg", "*.jpg", "*.insp", "*.tiff", "*.tif",
             "*.exr", "*.dng", "*.cr2", "*.nef", "*.raw", "*.npy"]


def main():
    """
    Entry point for the 'imsize' command-line application.
    """
    show_all = argv.exists("--all")
    verbose = not argv.exists("--quiet")
    show_help = argv.exists("--help")
    argv.exitIfAnyUnparsedOptions()
    if show_help:
        print("Usage: imsize [options] [file-or-directory ...]")
        print()
        print("  Displays the dimensions and uncompressed sizes of the given images.")
        print("  Runs very fast because only the header part of a file is read from")
        print("  disk (except for headerless Camera RAW and Nikon NEF).")
        print()
        print("  options:")
        print("    --all               show all extracted per-image metadata")
        print("    --quiet             do not show per-image information")
        print("    --help              show this help message")
        print()
        print("  example:")
        print("    imsize ~/Pictures")
        print()
        print("  supported file types:")
        print("   ", '\n    '.join(FILETYPES))
        print()
        sys.exit(-1)
    else:
        if verbose:
            print("See 'imsize --help' for command-line options.")
        paths = sys.argv[1:] or ["."]  # scan current directory if no arguments
        filespecs = find_files(paths)
        scan_sizes(filespecs, verbose, show_all)


def find_files(paths):
    """
    Collects all files with known filetypes from the given list of directories.
    """
    allfiles = []
    for path in paths:
        if os.path.isdir(path):
            for filetype in FILETYPES:
                allfiles += glob.glob(os.path.join(path, filetype))
                allfiles += glob.glob(os.path.join(path, filetype.upper()))
        elif os.path.isfile(path):
            allfiles += [path]
    return allfiles


def scan_sizes(filespecs, verbose, show_all):
    """
    Displays the dimensions of the given list of images.
    """
    total_compressed = 0
    total_uncompressed = 0
    num_processed = 0
    for filespec in sorted(filespecs):
        basename = os.path.basename(filespec)
        try:
            info = imsize.read(filespec)
        except RuntimeError as e:
            print(f"{basename}: Skipping: {e}")
            continue
        if info is None:
            print(f"{basename}: Unable to guess dimensions. Maybe not an image? Skipping.")
        else:
            num_processed += 1
            total_uncompressed += info.nbytes / 1024**2
            total_compressed += info.filesize / 1024**2
            if verbose:
                megs = info.nbytes / 1024**2
                mpix = info.width * info.height / 1000000
                est = " [estimated]" if info.uncertain else ""
                if info.rot90_ccw_steps in [0, 2]:
                    width, height = (info.width, info.height)
                else:
                    width, height = (info.height, info.width)
                print(f"{basename}: {width} x {height} x {info.nchan} x {info.bitdepth} bits => {megs:.1f} MB{est}, {mpix:.1f} MP")
            if show_all:
                print(info)
    print(f"Scanned {num_processed} images, total {total_compressed:.1f} MB compressed, {total_uncompressed:.1f} MB uncompressed")


if __name__ == "__main__":
    main()



"""
A minimal header parser for PNM/PGM/PPM images, meant solely for querying the
dimensions and bit depth.
"""

import re  # built-in library


######################################################################################
#
#  P U B L I C   A P I
#
######################################################################################

def dims(filespec, verbose=False):
    """
    Returns the dimensions (width, height, and number of channels) of the given
    PNM/PGM/PPM file. Also returns the maximum representable value of a pixel
    (typically 255, 1023, 4095, or 65535).
    """
    with open(filespec, "rb") as f:
        header = f.read(64)  # should be enough for any valid header
        shape, maxval = __parse_header(header, filespec, verbose)
        return (shape, maxval)


######################################################################################
#
#  I N T E R N A L   F U N C T I O N S
#
######################################################################################


def __parse_header(header, filespec, verbose=False):
    regex_pnm_header = b"(^(P[56])\\s+(\\d+)\\s+(\\d+)\\s+(\\d+)\\s)"
    match = re.search(regex_pnm_header, header)
    if match is not None:
        header, typestr, width, height, maxval = match.groups()
        width, height, maxval = int(width), int(height), int(maxval)
        numch = 3 if typestr == b"P6" else 1
        shape = (height, width, numch) if typestr == b"P6" else (height, width)
        if verbose:
            print(f"Reading file {filespec} ", end='')
            print(f"(w={width}, h={height}, c={numch}, maxval={maxval})")
        return (shape, maxval)
    raise RuntimeError(f"File {filespec} is not a valid PNM/PGM/PPM file.")



"""
Extracts image dimensions, bit depth, and other basic metadata.

Example:
  info = imsize.read("myfile.jpg")
  factor = info.nbytes / info.filesize
  print(f"{info.filespec}: compression factor = {factor.1f}")

https://github.com/toaarnio/imsize
"""

from .imsize import read, ImageInfo

__version__ = "1.3.1"
__all__ = ["read", "ImageInfo"]
